# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Alorese Corpus is a collection of language data in a couple of Alorese variation (Alor and Pantar Alorese). The collection is available in video, audio, and text formats with genres
ranging from Experiment or task, Stimuli, Discourse, and Written materials.
"""
import xml.etree.ElementTree as ET
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.sea_datasets.alorese.alorese_url import _URLS_DICT
from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@article{Moro2018-ms,
  title     = "The plural word hire in alorese: Contact-induced change from
               neighboring Alor-pantar languages",
  author    = "Moro, Francesca R",
  journal   = "Oceanic Linguistics",
  publisher = "University of Hawai'i Press",
  volume    =  57,
  number    =  1,
  pages     = "177--198",
  year      =  2018,
  language  = "en"
}
"""

_DATASETNAME = "alorese"
_DESCRIPTION = """\
 Alorese Corpus is a collection of language data in a couple of Alorese variation (Alor and Pantar Alorese). The collection is available in video, audio, and text formats with genres
 ranging from Experiment or task, Stimuli, Discourse, and Written materials.
"""
_HOMEPAGE = "https://hdl.handle.net/1839/e10d7de5-0a6d-4926-967b-0a8cc6d21fb1"
_LANGUAGES = ["aol", "ind"]
_LICENSE = Licenses.UNKNOWN.value
_LOCAL = False

_URLS = _URLS_DICT

_SUPPORTED_TASKS = [Tasks.SPEECH_RECOGNITION, Tasks.MACHINE_TRANSLATION]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class AloreseDataset(datasets.GeneratorBasedBuilder):
    """Alorese Corpus is a collection of language data in a couple of Alorese variation (Alor and Pantar Alorese). The collection is available in video, audio, and text formats with genres ranging
    from Experiment or task, Stimuli, Discourse, and Written materials."""

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=datasets.Version(_SOURCE_VERSION),
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}"
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_t2t",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"{_DATASETNAME} SEACrowd for text2text schema",
            schema="seacrowd_t2t",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_sptext",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"{_DATASETNAME} SEACrowd for sptext schema",
            schema="seacrowd_sptext",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "nr": datasets.Value("int64"),
                    "media_id": datasets.Value("string"),
                    "speaker_id": datasets.Value("string"),
                    "audio": datasets.Audio(sampling_rate=16000),
                    "annotation_aol": datasets.Value("string"),
                    "annotation_ind": datasets.Value("string"),
                    "begin_time": datasets.Value("int64"),
                    "end_time": datasets.Value("int64"),
                }
            )

        elif self.config.schema == "seacrowd_sptext":
            features = schemas.speech_text_features

        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        else:
            raise ValueError(f"Invalid config schema: {self.config.schema}")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:

        if self.config.schema == "seacrowd_t2t":
            filepath = {k: v["text_path"] for k, v in _URLS.items()}
            paths = dl_manager.download(filepath)
        else:
            paths = dl_manager.download(_URLS)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": paths,
                },
            ),
        ]

    def _generate_examples(self, filepath) -> Tuple[int, Dict]:

        if self.config.schema == "source":
            source_df = self._get_source_df(filepath)

            for k, row in source_df.iterrows():
                yield k, {
                    "nr": k + 1,
                    "media_id": row["media_id"],
                    "speaker_id": row["speaker_id"],
                    "audio": row["audio_path"],
                    "annotation_aol": row["annotation_aol"],
                    "annotation_ind": row["annotation_ind"],
                    "begin_time": row["begin_time"],
                    "end_time": row["end_time"],
                }

        elif self.config.schema == "seacrowd_t2t":
            caption_df = self._merge_text_dfs(filepath)

            for k, row in caption_df.iterrows():
                yield k, {
                    "id": k + 1,
                    "text_1": row["annotation_aol"],
                    "text_2": row["annotation_ind"],
                    "text_1_name": _LANGUAGES[0],
                    "text_2_name": _LANGUAGES[1],
                }

        elif self.config.schema == "seacrowd_sptext":
            sptext_df = self._get_sptext_df(filepath)

            for k, row in sptext_df.iterrows():
                yield k, {
                    "id": k + 1,
                    "path": row["audio_path"],
                    "audio": row["audio_path"],
                    "text": row["annotation_aol"],
                    "speaker_id": row["speaker_id"],
                    "metadata": {
                        "speaker_age": None,
                        "speaker_gender": None
                }}

    def _get_time_df(self, xml_tree) -> pd.DataFrame:
        time_slot_values = [(time_slot.attrib["TIME_SLOT_ID"], int(time_slot.attrib["TIME_VALUE"])) for time_slot in xml_tree.iter(tag="TIME_SLOT")]

        return pd.DataFrame({"time_slot_id": [v[0] for v in time_slot_values], "time_value": [v[1] for v in time_slot_values]})

    def _get_aol_annotations(self, xml_tree) -> pd.DataFrame:
        aol_annotations = [(annotation.attrib["ANNOTATION_ID"], annotation.attrib["TIME_SLOT_REF1"], annotation.attrib["TIME_SLOT_REF2"], annotation.find("ANNOTATION_VALUE").text) for annotation in xml_tree.iter(tag="ALIGNABLE_ANNOTATION")]

        return pd.DataFrame({"annotation_id": [v[0] for v in aol_annotations], "time_slot_ref1": [v[1] for v in aol_annotations], "time_slot_ref2": [v[2] for v in aol_annotations], "annotation_value": [v[3] for v in aol_annotations]})

    def _get_ind_annotations(self, xml_tree) -> pd.DataFrame:
        ind_annotations = [(annotation.attrib["ANNOTATION_ID"], annotation.attrib["ANNOTATION_REF"], annotation.find("ANNOTATION_VALUE").text) for annotation in xml_tree.iter(tag="REF_ANNOTATION")]

        return pd.DataFrame({"annotation_id": [v[0] for v in ind_annotations], "annotation_ref_id": [v[1] for v in ind_annotations], "annotation_value": [v[2] for v in ind_annotations]})

    def _get_text_df(self, xml_tree) -> pd.DataFrame:
        time_df = self._get_time_df(xml_tree)
        aol_df = self._get_aol_annotations(xml_tree)
        ind_df = self._get_ind_annotations(xml_tree)

        df1 = aol_df.merge(time_df, left_on="time_slot_ref1", right_on="time_slot_id", how="left").rename(columns={"time_value": "begin_time", "annotation_value": "annotation_aol"}).drop(columns=["time_slot_ref1", "time_slot_id"])
        df2 = df1.merge(time_df, left_on="time_slot_ref2", right_on="time_slot_id", how="left").rename(columns={"time_value": "end_time"}).drop(columns=["time_slot_ref2", "time_slot_id"])
        final_df = df2.merge(ind_df, left_on="annotation_id", right_on="annotation_ref_id", how="left").rename(columns={"annotation_value": "annotation_ind"}).drop(columns=["annotation_ref_id", "annotation_id_y", "annotation_id_x"])

        return final_df[["annotation_aol", "annotation_ind", "begin_time", "end_time"]]

    def _merge_text_dfs(self, xml_dict) -> pd.DataFrame:
        final_df = pd.DataFrame()
        len_tracker = []
        media_ids = []

        xml_trees = [ET.parse(xml_path) for xml_path in xml_dict.values()]
        for xml_tree in xml_trees:
            cur_df = self._get_text_df(xml_tree)
            final_df = pd.concat([final_df, cur_df], axis=0)
            len_tracker.append(len(cur_df))

        media_id_list = list(xml_dict.keys())
        for i in range(len(len_tracker)):
            media_ids.extend([media_id_list[i]] * len_tracker[i])

        final_df["media_id"] = media_ids

        return final_df.reset_index()

    def _groupby_caption_by_media_ids(self, caption_df: pd.DataFrame) -> pd.DataFrame:
        caption_df = (
            caption_df.groupby("media_id")
            .agg({"annotation_aol": lambda x: " ".join([str(value) if value is not None else "<NONE>" for value in x]), "annotation_ind": lambda x: " ".join([str(value) if value is not None else "<NONE>" for value in x])})
            .reset_index()
        )
        return caption_df

    def _get_sptext_df(self, complete_dict) -> pd.DataFrame:
        xml_dict = {k: v["text_path"] for k, v in complete_dict.items()}

        audio_df = pd.DataFrame({"media_id": [k for k in complete_dict.keys()], "speaker_id": [k.split("_")[-1] for k in complete_dict.keys()], "audio_path": [v["audio_path"] for v in complete_dict.values()]})
        caption_df = self._groupby_caption_by_media_ids(self._merge_text_dfs(xml_dict))

        df = caption_df.merge(audio_df, on="media_id", how="inner")

        return df[["media_id", "speaker_id", "audio_path", "annotation_aol", "annotation_ind"]]

    def _get_source_df(self, complete_dict) -> pd.DataFrame:
        xml_dict = {k: v["text_path"] for k, v in complete_dict.items()}

        audio_df = pd.DataFrame({"media_id": [k for k in complete_dict.keys()], "speaker_id": [k.split("_")[-1] for k in complete_dict.keys()], "audio_path": [v["audio_path"] for v in complete_dict.values()]})
        text_df = self._merge_text_dfs(xml_dict)

        df = text_df.merge(audio_df, on="media_id", how="inner")

        return df[["media_id", "speaker_id", "audio_path", "annotation_aol", "annotation_ind", "begin_time", "end_time"]]



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import re
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import TASK_TO_SCHEMA, Licenses, Tasks

_CITATION = """\
@article{gonzales_corpus_2021,
    title = {The {Corpus} of {Singapore} {English} {Messages} ({CoSEM})},
    issn = {0883-2919, 1467-971X},
    url = {https://onlinelibrary.wiley.com/doi/10.1111/weng.12534},
    doi = {10.1111/weng.12534},
    language = {en},
    urldate = {2022-02-19},
    journal = {World Englishes},
    author = {Gonzales, Wilkinson Daniel Wong and Hiramoto, Mie and R. E. Leimgruber, Jakob and Lim, Jun Jie},
    month = feb,
    year = {2021},
}
"""

_DATASETNAME = "cosem"

_DESCRIPTION = """\
The CoSEM dataset consists of over 900,000 lines of online messages from the messaging platform WhatsApp collected from personal chat
logs of students enrolled in an advanced sociolinguistics class from the National University of Singapore. Messages collected were
from 2016 to 2019. The dataset is in .txt format, where each line of utterance is tagged with a unique identifier that includes its
metadata such as line number, year message was sent, and age and nationality of sender.
"""

_HOMEPAGE = "https://github.com/wdwgonzales/CoSEM/blob/main/Corpus/COSEM_v4_publicrelease_SEP172023.zip"

_LANGUAGES = ["eng"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)

_LICENSE = Licenses.CC0_1_0.value

_LOCAL = False

_URLS = {_DATASETNAME: "https://github.com/wdwgonzales/CoSEM/raw/main/Corpus/COSEM_v4_publicrelease_SEP172023.zip"}

_SUPPORTED_TASKS = [Tasks.SELF_SUPERVISED_PRETRAINING]
_SUPPORTED_SCHEMA_STRINGS = [f"seacrowd_{str(TASK_TO_SCHEMA[task]).lower()}" for task in _SUPPORTED_TASKS]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class CoSEMDataset(datasets.GeneratorBasedBuilder):
    """The CoSEM dataset consists of over 900,000 lines of online messages from the messaging platform WhatsApp collected from
    personal chat logs of students enrolled in an advanced sociolinguistics class from the National University of Singapore."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    subset_id = _DATASETNAME

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{subset_id}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=subset_id,
        )
    ]

    seacrowd_schema_config: list[SEACrowdConfig] = []

    for seacrowd_schema in _SUPPORTED_SCHEMA_STRINGS:

        seacrowd_schema_config.append(
            SEACrowdConfig(
                name=f"{subset_id}_{seacrowd_schema}",
                version=SEACROWD_VERSION,
                description=f"{_DATASETNAME} {seacrowd_schema} schema",
                schema=f"{seacrowd_schema}",
                subset_id=subset_id,
            )
        )

    BUILDER_CONFIGS.extend(seacrowd_schema_config)

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "text": datasets.Value("string"),
                }
            )

        elif self.config.schema == f"seacrowd_{str(TASK_TO_SCHEMA[Tasks.SELF_SUPERVISED_PRETRAINING]).lower()}":
            features = schemas.ssp_features

        else:
            raise ValueError(f"Invalid config: {self.config.name}")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        split_generators = []

        path = dl_manager.download_and_extract(_URLS[_DATASETNAME])

        split_generators.append(
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "path": os.path.join(path, "COSEM_v4_publicrelease_SEP172023"),
                },
            )
        )

        return split_generators

    def _generate_examples(self, path: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        files = os.listdir(path)
        file_paths = [os.path.join(path, file) for file in files]
        pattern = r"<(COSEM:.*?)>(.*?)(?=<COSEM:|$)"

        s = {}

        for file_path in file_paths:
            with open(file_path, "r", encoding="utf-8") as file:
                text = file.read()

                matches = re.findall(pattern, text, re.DOTALL)
                for match in matches:
                    key = match[0].strip()
                    value = match[1].strip()

                    if key in s:
                        continue
                    s[key] = value

                    if self.config.schema == "source" or self.config.schema == f"seacrowd_{str(TASK_TO_SCHEMA[Tasks.SELF_SUPERVISED_PRETRAINING]).lower()}":
                        yield key, {"id": key, "text": value}

                    else:
                        raise ValueError(f"Invalid config: {self.config.name}")



import csv
import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """
@article{Cabasag-2019-hate-speech,
  title={Hate speech in Philippine election-related tweets: Automatic detection and classification using natural language processing.},
  author={Neil Vicente Cabasag, Vicente Raphael Chan, Sean Christian Lim, Mark Edward Gonzales, and Charibeth Cheng},
  journal={Philippine Computing Journal},
  volume={XIV},
  number={1},
  month={August},
  year={2019}
}
"""

_DATASETNAME = "filipino_hatespeech_election"

_DESCRIPTION = """
The dataset used in this study was a subset of the corpus 1,696,613 tweets crawled by Andrade et al. and posted from November 2015 to May 2016 during the campaign period for the Philippine presidential election. They were culled
based on the presence of candidate names (e.g., Binay, Duterte, Poe, Roxas, and Santiago) and election-related hashtags (e.g., #Halalan2016, #Eleksyon2016, and #PiliPinas2016). Data preprocessing was performed to prepare the
tweets for feature extraction and classification. It consisted of the following steps: data de-identification, uniform resource locator (URL) removal, special character processing, normalization, hashtag processing, and tokenization.
"""

_HOMEPAGE = "https://huggingface.co/datasets/hate_speech_filipino"

_LANGUAGES = ["fil"]

_LICENSE = Licenses.UNKNOWN.value

_LOCAL = False

_URLS = {_DATASETNAME: "https://s3.us-east-2.amazonaws.com/blaisecruz.com/datasets/hatenonhate/hatespeech_raw.zip"}

_SUPPORTED_TASKS = [Tasks.ABUSIVE_LANGUAGE_PREDICTION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"

_CLASSES = ["0", "1"]  # corresponds to  ["non-hate-containing", "hate-containing"]


class FilipinoHatespeechElectionDataset(datasets.GeneratorBasedBuilder):
    """Hate Speech Text Classification Dataset in Filipino."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_text",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_text",
            subset_id=_DATASETNAME,
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        features = schemas.text_features(label_names=_CLASSES)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[_DATASETNAME]
        data_dir = dl_manager.download_and_extract(urls)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "hatespeech", "train.csv"),
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "hatespeech", "test.csv"),
                    "split": "test",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "hatespeech", "valid.csv"),
                    "split": "dev",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        with open(filepath, encoding="utf-8") as csv_file:
            csv_reader = csv.reader(csv_file, quotechar='"', delimiter=",", quoting=csv.QUOTE_ALL, skipinitialspace=True)
            next(csv_reader)
            for i, row in enumerate(csv_reader):
                try:
                    text, label = row
                    yield i, {"id": str(i), "text": row[0], "label": _CLASSES[int(row[1].strip()) - 1]}
                except ValueError:
                    pass



# coding=utf-8
import json
import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@inproceedings{tran2021vivqa,
  title={ViVQA: Vietnamese visual question answering},
  author={Tran, Khanh Quoc and Nguyen, An Trong and Le, An Tran-Hoai and Van Nguyen, Kiet},
  booktitle={Proceedings of the 35th Pacific Asia Conference on Language, Information and Computation},
  pages={683--691},
  year={2021}
}
"""
_DATASETNAME = "openvivqa"
_DESCRIPTION = """\
OpenViVQA (Open-domain Vietnamese Visual Question Answering) is a dataset for VQA (Visual Question Answering) with
open-ended answers in Vietnamese. It consisted of 11199 images associated with 37914 question-answer pairs (QAs).
Images in the OpenViVQA dataset are captured in Vietnam and question-answer pairs are created manually by Vietnamese
crowd workers.
"""
_HOMEPAGE = "https://huggingface.co/datasets/uitnlp/OpenViVQA-dataset"
_LANGUAGES = ["vie"]
_LICENSE = Licenses.MIT.value
_LOCAL = False
_HF_URL = "https://huggingface.co/datasets/uitnlp/OpenViVQA-dataset"
_URLS = {
    "dataset": {
        "train": "https://huggingface.co/datasets/uitnlp/OpenViVQA-dataset/raw/main/vlsp2023_train_data.json",
        "test": "https://huggingface.co/datasets/uitnlp/OpenViVQA-dataset/raw/main/vlsp2023_test_data.json",
        "dev": "https://huggingface.co/datasets/uitnlp/OpenViVQA-dataset/raw/main/vlsp2023_dev_data.json",
    },
    "images": {
        "train": "https://huggingface.co/datasets/uitnlp/OpenViVQA-dataset/resolve/main/train-images.zip?download=true",
        "test": "https://huggingface.co/datasets/uitnlp/OpenViVQA-dataset/resolve/main/test-images.zip?download=true",
        "dev": "https://huggingface.co/datasets/uitnlp/OpenViVQA-dataset/resolve/main/dev-images.zip?download=true",
    },
}
_SUPPORTED_TASKS = [Tasks.VISUAL_QUESTION_ANSWERING]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class OpenViVQADataset(datasets.GeneratorBasedBuilder):

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_imqa",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_imqa",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features({"img_path": datasets.Value("string"),
                                          "question": datasets.Value("string"),
                                          "answer": datasets.Value("string"),
                                          "id": datasets.Value("string")})
        elif self.config.schema == "seacrowd_imqa":
            features = schemas.imqa_features
            # features["meta"] = {"image_path": datasets.Value("string")}
        else:
            raise ValueError(f"No schema matched for {self.config.schema}")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        data_dir = dl_manager.download_and_extract(_URLS["dataset"])
        image_dir = dl_manager.download_and_extract(_URLS["images"])

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_dir["train"],
                    "imagepath": os.path.join(image_dir["train"], "training-images"),
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": data_dir["test"],
                    "imagepath": os.path.join(image_dir["test"], "test-images"),
                    "split": "test",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": data_dir["dev"],
                    "imagepath": os.path.join(image_dir["dev"], "dev-images"),
                    "split": "validation",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, imagepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        raw_examples = json.load(open(filepath, "r"))
        images = raw_examples["images"]
        data_annotations = raw_examples["annotations"]
        for sample_id, q_key in enumerate(list(data_annotations.keys())):
            quest_id = q_key
            sample = data_annotations[q_key]
            sample_img_id = sample["image_id"]
            sample_img_name = images[str(sample_img_id)]
            sample_img_path = os.path.join(imagepath, sample_img_name)
            sample_question = sample["question"]
            sample_answer = sample["answer"]
            if self.config.schema == "source":
                example = {
                    "img_path": sample_img_path,
                    "question": sample_question,
                    "answer": sample_answer,
                    "id": quest_id,
                }
            elif self.config.schema == "seacrowd_imqa":
                example = {
                    "id": q_key,
                    "question_id": q_key,
                    "document_id": q_key,
                    "questions": [sample_question],
                    "type": None,
                    "choices": None,
                    "context": sample_img_id,
                    "answer": [sample_answer],
                    "image_paths": [sample_img_path],
                    "meta": {},
                }
            yield sample_id, example



# Some code referenced from https://huggingface.co/datasets/Babelscape/SREDFM/blob/main/SREDFM.py

from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import jsonlines

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@inproceedings{huguet-cabot-et-al-2023-redfm-dataset,
    title = "RED$^{\rm FM}$: a Filtered and Multilingual Relation Extraction Dataset",
    author = "Huguet Cabot, Pere-Lluís  and Tedeschi, Simone and Ngonga Ngomo, Axel-Cyrille and
      Navigli, Roberto",
    booktitle = "Proc. of the 61st Annual Meeting of the Association for Computational Linguistics: ACL 2023",
    month = jul,
    year = "2023",
    address = "Toronto, Canada",
    publisher = "Association for Computational Linguistics",
    url = "https://arxiv.org/abs/2306.09802",
}
"""

_DATASETNAME = "sredfm"


_DESCRIPTION = """\
SREDFM is an automatically annotated dataset for relation extraction task covering 18 languages, 400 relation types, 13 entity types, totaling more than 40 million triplet instances. SREDFM includes Vietnamnese.
"""

_HOMEPAGE = "https://github.com/babelscape/rebel"

_LANGUAGES = ["vie"]

_LICENSE = Licenses.CC_BY_SA_4_0.value

_LOCAL = False

_URLS = {
    "train": "https://huggingface.co/datasets/Babelscape/SREDFM/resolve/main/data/train.vi.jsonl",
    "dev": "https://huggingface.co/datasets/Babelscape/SREDFM/resolve/main/data/dev.vi.jsonl",
    "test": "https://huggingface.co/datasets/Babelscape/SREDFM/resolve/main/data/test.vi.jsonl",
    "relations_url": "https://huggingface.co/datasets/Babelscape/SREDFM/raw/main/relations.tsv",
}

_SUPPORTED_TASKS = [Tasks.RELATION_EXTRACTION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class SREDFMDataset(datasets.GeneratorBasedBuilder):
    """SREDFM is an automatically annotated dataset for relation extraction task.
    Relation Extraction (RE) is a task that identifies relationships between entities in a text,
    enabling the acquisition of relational facts and bridging the gap between natural language
    and structured knowledge. SREDFM covers 400 relation types, 13 entity types,
    totaling more than 40 million triplet instances."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_kb",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_kb",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "docid": datasets.Value("string"),
                    "title": datasets.Value("string"),
                    "uri": datasets.Value("string"),
                    "text": datasets.Value("string"),
                    "entities": [
                        {
                            "uri": datasets.Value(dtype="string"),
                            "surfaceform": datasets.Value(dtype="string"),
                            "type": datasets.Value(dtype="string"),
                            "start": datasets.Value(dtype="int32"),
                            "end": datasets.Value(dtype="int32"),
                        }
                    ],
                    "relations": [
                        {
                            "subject": datasets.Value(dtype="int32"),
                            "predicate": datasets.Value(dtype="string"),
                            "object": datasets.Value(dtype="int32"),
                        }
                    ],
                }
            )

        elif self.config.schema == "seacrowd_kb":
            features = schemas.kb_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        data_dir = dl_manager.download_and_extract(_URLS)

        relation_names = dict()
        relation_path = data_dir["relations_url"]
        with open(relation_path, encoding="utf-8") as f:
            for row in f:
                rel_code, rel_name, _, _ = row.strip().split("\t")
                relation_names[rel_code] = rel_name

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": data_dir["train"], "relation_names": relation_names},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": data_dir["test"], "relation_names": relation_names},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": data_dir["dev"], "relation_names": relation_names},
            ),
        ]

    def _generate_examples(self, filepath: Path, relation_names: dict) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        if self.config.schema == "source":
            with jsonlines.open(filepath) as f:
                skip = set()
                for example in f.iter():
                    if example["docid"] in skip:
                        continue
                    skip.add(example["docid"])

                    entities = []
                    for entity in example["entities"]:
                        entities.append(
                            {
                                "uri": entity["uri"],
                                "surfaceform": entity["surfaceform"],
                                "start": entity["boundaries"][0],
                                "end": entity["boundaries"][1],
                                "type": entity["type"],
                            }
                        )

                    relations = []
                    for relation in example["relations"]:
                        if relation["predicate"]["uri"] not in relation_names or relation["confidence"] <= 0.75:
                            continue

                        relations.append(
                            {
                                "subject": entities.index(
                                    {
                                        "uri": relation["subject"]["uri"],
                                        "surfaceform": relation["subject"]["surfaceform"],
                                        "start": relation["subject"]["boundaries"][0],
                                        "end": relation["subject"]["boundaries"][1],
                                        "type": relation["subject"]["type"],
                                    }
                                ),
                                "predicate": relation_names[relation["predicate"]["uri"]],
                                "object": entities.index(
                                    {
                                        "uri": relation["object"]["uri"],
                                        "surfaceform": relation["object"]["surfaceform"],
                                        "start": relation["object"]["boundaries"][0],
                                        "end": relation["object"]["boundaries"][1],
                                        "type": relation["object"]["type"],
                                    }
                                ),
                            }
                        )

                    if len(relations) == 0:
                        continue

                    yield example["docid"], {
                        "docid": example["docid"],
                        "title": example["title"],
                        "uri": example["uri"],
                        "text": example["text"],
                        "entities": entities,
                        "relations": relations,
                    }

        elif self.config.schema == "seacrowd_kb":
            with jsonlines.open(filepath) as f:
                skip = set()
                i = 0
                for example in f.iter():
                    if example["docid"] in skip:
                        continue
                    skip.add(example["docid"])

                    i += 1
                    processed_text = example["text"].replace("\n", " ")
                    passages = [
                        {
                            "id": f"{i}-{example['uri']}",
                            "type": "text",
                            "text": [processed_text],
                            "offsets": [[0, len(processed_text)]],
                        }
                    ]

                    entities = []
                    for entity in example["entities"]:
                        entities.append(
                            {
                                "id": entity["uri"],
                                "type": entity["type"],
                                "text": [entity["surfaceform"]],
                                "offsets": [entity["boundaries"]],
                                "normalized": {"db_name": "", "db_id": ""},
                            }
                        )

                    relations = []
                    for relation in example["relations"]:
                        if relation["predicate"]["uri"] not in relation_names or relation["confidence"] <= 0.75:
                            continue

                        i += 1
                        sub = relation["subject"]
                        pred = relation["predicate"]
                        obj = relation["object"]
                        relations.append(
                            {
                                "id": f"{i}-{sub['uri']}-{pred['uri']}-{obj['uri']}",
                                "type": relation_names[pred["uri"]],
                                "arg1_id": str(
                                    entities.index(
                                        {
                                            "id": sub["uri"],
                                            "type": sub["type"],
                                            "text": [sub["surfaceform"]],
                                            "offsets": [sub["boundaries"]],
                                            "normalized": {"db_name": "", "db_id": ""},
                                        }
                                    )
                                ),
                                "arg2_id": str(
                                    entities.index(
                                        {
                                            "id": obj["uri"],
                                            "type": obj["type"],
                                            "text": [obj["surfaceform"]],
                                            "offsets": [obj["boundaries"]],
                                            "normalized": {"db_name": "", "db_id": ""},
                                        }
                                    )
                                ),
                                "normalized": {"db_name": "", "db_id": ""},
                            }
                        )

                    for entity in entities:
                        i += 1
                        entity["id"] = f"{i}-{entity['id']}"

                    if len(relations) == 0:
                        continue

                    yield example["docid"], {
                        "id": example["docid"],
                        "passages": passages,
                        "entities": entities,
                        "relations": relations,
                        "events": [],
                        "coreferences": [],
                    }



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import glob
import json
import os
from pathlib import Path
from typing import Dict, List, Tuple, Union

import datasets

from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

# no paper citation
_CITATION = """\
"""
_DATASETNAME = "thai_ser"
_DESCRIPTION = """\
THAI SER dataset consists of 5 main emotions assigned to actors: Neutral,
Anger, Happiness, Sadness, and Frustration. The recordings were 41 hours,
36 minutes long (27,854 utterances), and were performed by 200 professional
actors (112 female, 88 male) and directed by students, former alumni, and
professors from the Faculty of Arts, Chulalongkorn University. The THAI SER
contains 100 recordings and is separated into two main categories: Studio and
Zoom. Studio recordings also consist of two studio environments: Studio A, a
controlled studio room with soundproof walls, and Studio B, a normal room
without soundproof or noise control.
"""
_HOMEPAGE = "https://github.com/vistec-AI/dataset-releases/releases/tag/v1"
_LANGUAGES = ["tha"]
_LICENSE = Licenses.CC_BY_SA_4_0.value
_LOCAL = False

_URLS = {
    "actor_demography": "https://github.com/vistec-AI/dataset-releases/releases/download/v1/actor_demography.json",
    "emotion_label": "https://github.com/vistec-AI/dataset-releases/releases/download/v1/emotion_label.json",
    "studio": {
        "studio1-10": "https://github.com/vistec-AI/dataset-releases/releases/download/v1/studio1-10.zip",
        "studio11-20": "https://github.com/vistec-AI/dataset-releases/releases/download/v1/studio11-20.zip",
        "studio21-30": "https://github.com/vistec-AI/dataset-releases/releases/download/v1/studio21-30.zip",
        "studio31-40": "https://github.com/vistec-AI/dataset-releases/releases/download/v1/studio31-40.zip",
        "studio41-50": "https://github.com/vistec-AI/dataset-releases/releases/download/v1/studio41-50.zip",
        "studio51-60": "https://github.com/vistec-AI/dataset-releases/releases/download/v1/studio51-60.zip",
        "studio61-70": "https://github.com/vistec-AI/dataset-releases/releases/download/v1/studio61-70.zip",
        "studio71-80": "https://github.com/vistec-AI/dataset-releases/releases/download/v1/studio71-80.zip",
    },
    "zoom": {"zoom1-10": "https://github.com/vistec-AI/dataset-releases/releases/download/v1/zoom1-10.zip", "zoom11-20": "https://github.com/vistec-AI/dataset-releases/releases/download/v1/zoom11-20.zip"},
}
_URLS["studio_zoom"] = {**_URLS["studio"], **_URLS["zoom"]}

_SUPPORTED_TASKS = [Tasks.SPEECH_EMOTION_RECOGNITION]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class ThaiSER(datasets.GeneratorBasedBuilder):
    """Thai speech emotion recognition dataset THAI SER contains 100 recordings (80 studios and 20 zooms)."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    SEACROWD_SCHEMA_NAME = "speech"
    _LABELS = ["Neutral", "Angry", "Happy", "Sad", "Frustrated"]

    BUILDER_CONFIGS = [
        # studio
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=f"{_DATASETNAME}",
        ),
        # studio and zoom
        SEACrowdConfig(
            name=f"{_DATASETNAME}_include_zoom_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}_include_zoom",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_include_zoom_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=f"{_DATASETNAME}_include_zoom",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "path": datasets.Value("string"),
                    "audio": datasets.Audio(sampling_rate=44_100),
                    "speaker_id": datasets.Value("string"),
                    "labels": datasets.ClassLabel(names=self._LABELS),
                    "majority_emo": datasets.Value("string"),  # 'None' when no single majority
                    "annotated": datasets.Value("string"),
                    "agreement": datasets.Value("float32"),
                    "metadata": {
                        "speaker_age": datasets.Value("int64"),
                        "speaker_gender": datasets.Value("string"),
                    },
                }
            )
        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            # same as schemas.speech_features(self._LABELS) except for sampling_rate
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "path": datasets.Value("string"),
                    "audio": datasets.Audio(sampling_rate=44_100),
                    "speaker_id": datasets.Value("string"),
                    "labels": datasets.ClassLabel(names=self._LABELS),
                    "metadata": {
                        "speaker_age": datasets.Value("int64"),
                        "speaker_gender": datasets.Value("string"),
                    },
                }
            )

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        setting = "studio_zoom" if "zoom" in self.config.name else "studio"

        data_paths = {"actor_demography": Path(dl_manager.download_and_extract(_URLS["actor_demography"])), "emotion_label": Path(dl_manager.download_and_extract(_URLS["emotion_label"])), setting: {}}
        for url_name, url_path in _URLS[setting].items():
            data_paths[setting][url_name] = Path(dl_manager.download_and_extract(url_path))

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "actor_demography_filepath": data_paths["actor_demography"],
                    "emotion_label_filepath": data_paths["emotion_label"],
                    "data_filepath": data_paths[setting],
                    "split": "train",
                },
            )
        ]

    def _generate_examples(self, actor_demography_filepath: Path, emotion_label_filepath: Path, data_filepath: Dict[str, Union[Path, Dict]], split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        # read actor_demography file
        with open(actor_demography_filepath, "r", encoding="utf-8") as actor_demography_file:
            actor_demography = json.load(actor_demography_file)
        actor_demography_dict = {actor["Actor's ID"]: {"speaker_age": actor["Age"], "speaker_gender": actor["Sex"].lower()} for actor in actor_demography["data"]}

        # read emotion_label file
        with open(emotion_label_filepath, "r", encoding="utf-8") as emotion_label_file:
            emotion_label = json.load(emotion_label_file)

        # iterate through data folders
        for folder_path in data_filepath.values():
            flac_files = glob.glob(os.path.join(folder_path, "**/*.flac"), recursive=True)
            # iterate through recordings
            for audio_path in flac_files:
                id = audio_path.split("/")[-1]
                speaker_id = id.split("_")[2].strip("actor")
                # labels in emotion_label are incomplete, labels only provided for microphone types: mic, con
                # otherwise, obtain label from id for scripted utterances and skip sample for the improvised utterances
                if id in emotion_label.keys():
                    assigned_emo = emotion_label[id][0]["assigned_emo"]
                    majority_emo = emotion_label[id][0]["majority_emo"]
                    agreement = emotion_label[id][0]["agreement"]
                    annotated = emotion_label[id][0]["annotated"]
                else:
                    if "script" in id:
                        label = id.split("_")[-1][0]  # Emotion (1 = Neutral, 2 = Angry, 3 = Happy, 4 = Sad, 5 = Frustrated)
                        assigned_emo = self._LABELS[int(label) - 1]
                        majority_emo = agreement = annotated = None
                    else:
                        continue

                if self.config.schema == "source":
                    example = {
                        "id": id.strip(".flac"),
                        "path": audio_path,
                        "audio": audio_path,
                        "speaker_id": speaker_id,
                        "labels": assigned_emo,
                        "majority_emo": majority_emo,
                        "agreement": agreement,
                        "annotated": annotated,
                        "metadata": {"speaker_age": actor_demography_dict[speaker_id]["speaker_age"], "speaker_gender": actor_demography_dict[speaker_id]["speaker_gender"]},
                    }
                elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
                    example = {
                        "id": id.strip(".flac"),
                        "path": audio_path,
                        "audio": audio_path,
                        "speaker_id": speaker_id,
                        "labels": assigned_emo,
                        "metadata": {"speaker_age": actor_demography_dict[speaker_id]["speaker_age"], "speaker_gender": actor_demography_dict[speaker_id]["speaker_gender"]},
                    }

                yield id.strip(".flac"), example



_URLS_DICT = {
    "LIO": {
        "LIO_ave_maria_transcription":{
            "text_path": "https://archive.mpi.nl/tla/islandora/object/tla%3A1839_5fabcbda_6835_4ae3_b66f_21daae70d7b6/datastream/OBJ/download",
        },
        "LIO_positional_transcription":{
            "text_path": "https://archive.mpi.nl/tla/islandora/object/tla%3A1839_c1c1b5d6_d607_44c2_8f39_cda2041275aa/datastream/OBJ/download"
        },
        "LIO_frogstory_transcription":{
            "text_path": "https://archive.mpi.nl/tla/islandora/object/tla%3A1839_f290b09f_c351_46fe_8d86_d2ac624294c1/datastream/OBJ/download"
        },
        "LIO_frogstory_stan_jaya_transcription":{
            "text_path": "https://archive.mpi.nl/tla/islandora/object/tla%3A1839_1f24eace_b603_46f4_96d0_0054305caaf6/datastream/OBJ/download"
        },
        "LIO_history_transcription":{
            "text_path": "https://archive.mpi.nl/tla/islandora/object/tla%3A1839_b3b9b4de_a76e_440d_bf03_12c51ddc7954/datastream/OBJ/download"
        },
        "LIO_scorpion_story_transcription":{
            "text_path": "https://archive.mpi.nl/tla/islandora/object/tla%3A1839_d3b552ce_72d4_4d94_8510_8626b5cb91da/datastream/OBJ/download"
        },
        "LIO_our_father_transcription": {
             "text_path": "https://archive.mpi.nl/tla/islandora/object/tla%3A1839_50477a6a_12c6_4ae1_976a_9770f356062e/datastream/OBJ/download"
        },
        "LIO_credo_transcription": {
            "text_path": "https://archive.mpi.nl/tla/islandora/object/tla%3A1839_90f05ac6_813b_4570_b25c_f8d8b12bf8f0/datastream/OBJ/download"
        },
        "LIO_father_son_holy_spirit":{
            "text_path": "https://archive.mpi.nl/tla/islandora/object/tla%3A1839_9016339d_164f_464e_8653_9d941dd622b4/datastream/OBJ/download"
        },
        "LIO_glory_be_transcription": {
            "text_path": "https://archive.mpi.nl/tla/islandora/object/tla%3A1839_9a5f906c_bd51_40a3_a07a_39a99bb21d5a/datastream/OBJ/download"
        },
        "LIO_sambal_recipe_transcription": {
            "text_path": "https://archive.mpi.nl/tla/islandora/object/tla%3A1839_c2c00068_d01e_4ee3_877d_b42e1b0e77c5/datastream/OBJ/download"
        },
        "LIO_bridewealth_transcription": {
            "text_path": "https://archive.mpi.nl/tla/islandora/object/tla%3A1839_02e08440_721c_4a04_9ca5_7c9fb86eca2d/datastream/OBJ/download"
        },
        "LIO_gawi_kelimutu_transcription":{
            "text_path": "https://archive.mpi.nl/tla/islandora/object/tla%3A1839_d1e32c05_4e58_4171_9c59_0b394a13dc0b/datastream/OBJ/download"
        },
        "LIO_song_ine_transcription": {
            "text_path": "https://archive.mpi.nl/tla/islandora/object/tla%3A1839_d504e40b_e70e_4062_b406_8d69e162d392/datastream/OBJ/download"
        }
    },
    "ENDE": {
        "ENDE_positional_transcription": {
            "text_path": "https://archive.mpi.nl/tla/islandora/object/tla%3A1839_1655f827_74d6_47c9_83eb_c9be02a3ac3d/datastream/OBJ/download"
        },
        "ENDE_frogstory_transcription":{
            "text_path": "https://archive.mpi.nl/tla/islandora/object/tla%3A1839_b990239e_6229_4c58_96c3_71755040a8bb/datastream/OBJ/download"
        }
    },
    "NAGE":{
        "Nage_Ondorea_Barat_Lexirumah_Wordlist_Original":{
            "text_path": "https://archive.mpi.nl/tla/islandora/object/tla%3A1839_34f3575a_f83f_4674_a1ac_8abd0006e151/datastream/OBJ/download"
        }
    }
}


import os
from typing import Dict, List, Tuple

import datasets
import jsonlines as jl
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@inproceedings{thapliyal-etal-2022-crossmodal,
    title = "Crossmodal-3600: A Massively Multilingual Multimodal Evaluation Dataset",
    author = "Thapliyal, Ashish V.  and
      Pont Tuset, Jordi  and
      Chen, Xi  and
      Soricut, Radu",
    editor = "Goldberg, Yoav  and
      Kozareva, Zornitsa  and
      Zhang, Yue",
    booktitle = "Proceedings of the 2022 Conference on Empirical Methods in Natural Language Processing",
    month = dec,
    year = "2022",
    address = "Abu Dhabi, United Arab Emirates",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2022.emnlp-main.45",
    doi = "10.18653/v1/2022.emnlp-main.45",
    pages = "715--729",
}
"""

_DATASETNAME = "cc3m_35l"

_DESCRIPTION = """\
    CC3M-35L is created by translating Conceptual Captions 3M (Sharma et al., 2018),
    originally in English, to the other 34 languages using Google's machine translation API.
"""

_HOMEPAGE = "https://google.github.io/crossmodal-3600/"

_LICENSE = Licenses.CC_BY_4_0.value

# the image URLs are contained in tsv file together with the original captions which can be downloaded locally using google account.
# those tsv file originally can be found and downloaded from this page https://ai.google.com/research/ConceptualCaptions/download
# there are no direct image folder ready, so it needs to be downloaded one by one
# some warnings may occur when downloading due to reasons such as security certificate and others
_URLS = {
    "trans_train": "https://storage.googleapis.com/crossmodal-3600/cc3m_mt_train.jsonl.gz",
    "trans_dev": "https://storage.googleapis.com/crossmodal-3600/cc3m_mt_dev.jsonl.gz",
}

_SUPPORTED_TASKS = [Tasks.IMAGE_CAPTIONING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"

_LANGUAGES = ["fil", "ind", "tha", "vie"]

_LOCAL = True


class CC3M35L(datasets.GeneratorBasedBuilder):
    """
    CC3M-35L is created by translating Conceptual Captions 3M (Sharma et al., 2018),
    originally in English, to the other 34 languages using Google's machine translation API.
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [SEACrowdConfig(name=f"cc3m_35l_{lang}_source", version=datasets.Version(_SOURCE_VERSION), description=f"cc3m_35l_{lang} source schema", schema="source", subset_id=f"cc3m_35l_{lang}",) for lang in _LANGUAGES] + [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_{lang}_seacrowd_imtext",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"{_DATASETNAME}_{lang} SEACrowd schema",
            schema="seacrowd_imtext",
            subset_id=f"{_DATASETNAME}_{lang}",
        )
        for lang in _LANGUAGES
    ]

    DEFAULT_CONFIG_NAME = "cc3m_35l_id_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "image_paths": datasets.Value("string"),
                    "src_lang": datasets.Value("string"),
                    "caption_tokenized": datasets.Value("string"),
                    "trg_lang": datasets.Value("string"),
                    "translation_tokenized": datasets.Value("string"),
                    "backtranslation_tokenized": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_imtext":
            features = schemas.image_text_features()

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def fill_img_path(self, df: pd.DataFrame, line: dict):
        exceptions = []
        selected_row = df.query('caption==@line["caption_tokenized"]')
        # it may return several rows, skip of empty
        if not selected_row.empty:
            # for each row, download the image, use its path and put the translation
            for idx, row in selected_row.iterrows():
                row["trans_caption"] = line["translation_tokenized"]
                row["backtrans_caption"] = line["backtranslation_tokenized"]
                # if the image cannot be downloaded for some reason, skip it
                # may cause difference in the total data each run
                try:
                    row["img_path"] = datasets.DownloadManager().download(row["img_url"])
                except:
                    exceptions.append(idx)

        return selected_row, exceptions

    def is_target(self, line: dict, trg_lang: str):
        if line["trg_lang"] == trg_lang:
            return line

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        dev_path = dl_manager.download_and_extract(_URLS["trans_dev"])
        train_path = dl_manager.download_and_extract(_URLS["trans_train"])

        if self.config.data_dir is None:
            raise ValueError("This is a local dataset. Please pass the data_dir kwarg to load_dataset.")
        else:
            data_dir = self.config.data_dir

        # read tsv from local train and validation files
        gcc_val = os.path.join(data_dir, "Validation_GCC-1.1.0-Validation.tsv")
        gcc_train = os.path.join(data_dir, "Train_GCC-training.tsv")

        # make it into pandas dataframe
        colnames = ["caption", "img_url"]
        gcc_val_df = pd.read_csv(gcc_val, sep="\t", header=None, names=colnames)
        gcc_train_df = pd.read_csv(gcc_train, sep="\t", header=None, names=colnames)

        # add new column to keep the downloaded image path
        gcc_val_df["img_path"] = None
        gcc_train_df["img_path"] = None

        # add new column to keep the translated caption
        gcc_val_df["trans_caption"] = None
        gcc_train_df["trans_caption"] = None

        gcc_val_df["backtrans_caption"] = None
        gcc_train_df["backtrans_caption"] = None

        # match the original captions in the translated set to the dataframe caption
        # download the images from the URL and use it as the filepath
        train_exceptions = []
        val_exceptions = []

        current_lang = self.config.subset_id.split("_")[2]
        val_caption_targets = []
        train_caption_targets = []

        # filter validation data
        with jl.open(os.path.join(dev_path), mode="r") as j:
            val_caption_targets = [line for line in j if line["trg_lang"] == current_lang]
            
            #for line in val_caption_targets[:100]: # this was for debugging only
            for line in val_caption_targets:
                res = self.fill_img_path(gcc_train_df, line)
                val_exceptions.extend(res[1])
                gcc_val_df.update(res[0])
            
        # clean the memory
        val_caption_targets = []

        # filter train data
        with jl.open(os.path.join(train_path), mode="r") as j:
            train_caption_targets = [line for line in j if line["trg_lang"] == current_lang]
            
                
            #for line in train_caption_targets[:100]: # this was for debugging only
            for line in train_caption_targets:
                res = self.fill_img_path(gcc_val_df, line)
                train_exceptions.extend(res[1])
                gcc_train_df.update(res[0])

        # clean the memory
        train_caption_targets = []

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": gcc_train_df,
                    "exceptions": train_exceptions,
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": gcc_val_df,
                    "exceptions": val_exceptions,
                },
            ),
        ]

    def _generate_examples(self, filepath: dict, exceptions: list) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        for idx, row in filepath.iterrows():
            if idx not in exceptions:
                if self.config.schema == "source":
                    yield idx, {
                        "id": str(idx),
                        "image_paths": row["img_path"],
                        "src_lang": "en",
                        "caption_tokenized": row["caption"],
                        "trg_lang": self.config.subset_id.split("_")[2],
                        "translation_tokenized": row["trans_caption"],
                        "backtranslation_tokenized": row["backtrans_caption"],
                    }

                elif self.config.schema == "seacrowd_imtext":
                    yield idx, {
                        "id": str(idx),
                        "image_paths": [row["img_path"]],
                        "texts": row["trans_caption"],
                        "metadata": {
                            "context": None,
                            "labels": None,
                        },
                    }

                else:
                    raise ValueError(f"Invalid config: {self.config.name}")



import json
import os
from pathlib import Path
from typing import Dict, Generator, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@article{dac2023okapi,
  title={Okapi: Instruction-tuned Large Language Models in Multiple Languages with Reinforcement Learning from Human Feedback},
  author={Dac Lai, Viet and Van Nguyen, Chien and Ngo, Nghia Trung and Nguyen, Thuat and Dernoncourt, Franck and Rossi, Ryan A and Nguyen, Thien Huu},
  journal={arXiv e-prints},
  pages={arXiv--2307},
  year={2023}
}

@article{Clark2018ThinkYH,
  title={Think you have Solved Question Answering? Try ARC, the AI2 Reasoning Challenge},
  author={Peter Clark and Isaac Cowhey and Oren Etzioni and Tushar Khot and Ashish Sabharwal and Carissa Schoenick and Oyvind Tafjord},
  journal={ArXiv},
  year={2018},
  volume={abs/1803.05457}
}
"""

_DATASETNAME = "okapi_m_arc"

_DESCRIPTION = """\
mARC is a Multilingual translation of AI2's Arc Challenge from the paper "Okapi: Instruction-tuned Large Language Models in Multiple Languages with Reinforcement Learning from Human Feedback" (Lai et al., 2023).
The original ARC dataset is a multiple-choice question answering dataset of 7,787 genuine grade-school level science questions assembled to encourage research in advanced question-answering.
The dataset is partitioned into a Challenge Set and an Easy Set, where the former contains only questions answered incorrectly by both a retrieval-based algorithm and a word co-occurrence algorithm.
We also include a corpus of over 14 million science sentences relevant to the task and an implementation of three neural baseline models for this dataset. We pose ARC as a challenge to the community.
"""


_HOMEPAGE = "https://huggingface.co/datasets/jon-tow/okapi_arc_challenge"
_LICENSE = Licenses.CC_BY_NC_4_0.value
_LOCAL = False
_LANGUAGES = ["ind", "vie"]

_LANG_MAP = {"ind": "id", "vie": "vi"}
_URLS = {
    "base_url": "https://huggingface.co/datasets/jon-tow/okapi_arc_challenge/resolve/main",
}
_SUPPORTED_TASKS = [Tasks.QUESTION_ANSWERING]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class MultilingualArc(datasets.GeneratorBasedBuilder):
    """mARC is a Multilingual translation of AI2's Arc Challenge which is a multiple-choice question answering dataset
    of 7,787 genuine grade-school level science questions assembled to encourage research in advanced question-answering"""

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="okapi_m_arc_vie_source",
            version=datasets.Version(_SOURCE_VERSION),
            description="Vietnamese mARC source schema",
            schema="source",
            subset_id="okapi_m_arc_vie_source",
        ),
        SEACrowdConfig(
            name="okapi_m_arc_ind_source",
            version=datasets.Version(_SOURCE_VERSION),
            description="Indonesian mARC source schema",
            schema="source",
            subset_id="okapi_m_arc_ind_source",
        ),
        SEACrowdConfig(
            name="okapi_m_arc_vie_seacrowd_qa",
            version=datasets.Version(_SEACROWD_VERSION),
            description="Vietnamese mARC SEACrowd question answering schema",
            schema="seacrowd_qa",
            subset_id="okapi_m_arc_vie_seacrowd_qa",
        ),
        SEACrowdConfig(
            name="okapi_m_arc_ind_seacrowd_qa",
            version=datasets.Version(_SEACROWD_VERSION),
            description="Indonesian mARC SEACrowd question answering schema",
            schema="seacrowd_qa",
            subset_id="okapi_m_arc_ind_seacrowd_qa",
        ),
    ]

    DEFAULT_CONFIG_NAME = "okapi_m_arc_ind_seacrowd_qa"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "question": datasets.Value("string"),
                    "choices": datasets.features.Sequence(
                        {
                            "text": datasets.Value("string"),
                            "label": datasets.Value("string"),
                        }
                    ),
                    "answerKey": datasets.Value("string"),
                }
            )
        else:
            features = schemas.qa_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        lang = self.config.subset_id[: -(len(self.config.schema) + 1)].split("_")[-1]
        train_path = Path(dl_manager.download_and_extract(f"{_URLS['base_url']}/data/{_LANG_MAP[lang]}_train.json"))
        valid_path = Path(dl_manager.download_and_extract(f"{_URLS['base_url']}/data/{_LANG_MAP[lang]}_validation.json"))
        test_path = Path(dl_manager.download_and_extract(f"{_URLS['base_url']}/data/{_LANG_MAP[lang]}_test.json"))

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": train_path},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": valid_path},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": test_path},
            ),
        ]

    def _generate_examples(self, filepath) -> Generator[Tuple[int, Dict], None, None]:
        """Yields examples."""
        with open(filepath, encoding="utf-8") as f:
            data = json.load(f)

        for i, d in enumerate(data):
            text_choices = []
            label_choices = []
            if "option_a" in d:
                text_choices.append(d["option_a"])
                label_choices.append("A")
            if "option_b" in d:
                text_choices.append(d["option_b"])
                label_choices.append("B")
            if "option_c" in d:
                text_choices.append(d["option_c"])
                label_choices.append("C")
            if "option_d" in d:
                text_choices.append(d["option_d"])
                label_choices.append("D")
            if "option_e" in d:
                text_choices.append(d["option_e"])
                label_choices.append("E")

            if self.config.schema == "source":
                yield i, {
                    "id": d["id"],
                    "answerKey": d["answer"],
                    "question": d["instruction"],
                    "choices": {"text": text_choices, "label": label_choices},
                }
            else:
                yield i, {
                    "id": i,
                    "question_id": d["id"],
                    "document_id": d["id"],
                    "question": d["instruction"],
                    "type": "multiple_choice",
                    "choices": [f"{label}. {text}" for label, text in zip(label_choices, text_choices)],
                    "context": None,
                    "answer": [f'{d["answer"]}. {text_choices[ord(d["answer"])-65]}'],
                    "meta": {}
                }



import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import jsonlines

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@article{Lowphansirikul2021,
    author={Lowphansirikul, Lalita
            and Polpanumas, Charin
            and Rutherford, Attapol T.
            and Nutanong, Sarana},
    title={A large English--Thai parallel corpus from the web and machine-generated text},
    journal={Language Resources and Evaluation},
    year={2021},
    month={Mar},
    day={30},
    issn={1574-0218},
    doi={10.1007/s10579-021-09536-6},
    url={https://doi.org/10.1007/s10579-021-09536-6}
"""

_DATASETNAME = "generated_review_enth"

_DESCRIPTION = """\
Generated Reviews ENTH is created as part of scb-mt-en-th-2020 for machine translation task. This dataset (referred to as generated_reviews_yn in scb-mt-en-th-2020) are English product reviews generated by CTRL, translated by Google Translate API and annotated as accepted or rejected (correct) based on fluency and adequacy of the translation by human annotators. This allows it to be used for English-to-Thai translation quality esitmation (binary label), machine translation, and sentiment analysis. For SEACrowd, we use data with correct = 1.
"""

_HOMEPAGE = "https://github.com/vistec-ai/generated_reviews_enth"

_LANGUAGES = ["tha", "eng"]

_LICENSE = Licenses.CC_BY_SA_4_0.value

_LOCAL = False

_URLS = {_DATASETNAME: "https://github.com/vistec-AI/generated_reviews_enth/raw/main/data.zip"}

_SUPPORTED_TASKS = [Tasks.MACHINE_TRANSLATION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class GeneratedReviewENTHDataset(datasets.GeneratorBasedBuilder):
    """This dataset (referred to as generated_reviews_yn in scb-mt-en-th-2020) are English product reviews generated by CTRL, translated by Google Translate API and annotated as accepted or rejected (correct) based on fluency and adequacy of the translation by human annotators. This allows it to be used for English-to-Thai translation quality esitmation (binary label), machine translation, and sentiment analysis."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description="Generated Review EN-TH source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_t2t",
            version=SEACROWD_VERSION,
            description="Generated Review EN-TH SEACrowd schema",
            schema="seacrowd_t2t",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "en_segment": datasets.Value("string"),
                    "th_segment": datasets.Value("string"),
                    "review_star": datasets.Value("int32"),
                    "correct": datasets.Value("bool"),
                }
            )

        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        urls = _URLS[_DATASETNAME]
        data_dir = dl_manager.download_and_extract(urls)
        data_dir = os.path.join(data_dir, "data")

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "train.jsonl"),
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "test.jsonl"),
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "valid.jsonl"),
                },
            ),
        ]

    def _generate_examples(self, filepath: Path) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        if self.config.schema == "source":
            with jsonlines.open(filepath) as f:
                i = -1
                for example in f.iter():
                    i += 1
                    yield str(i), {
                        "en_segment": example["en_segment"],
                        "th_segment": example["th_segment"],
                        "review_star": example["review_star"],
                        "correct": example["correct"],
                    }

        elif self.config.schema == "seacrowd_t2t":
            with jsonlines.open(filepath) as f:
                i = -1
                for example in f.iter():
                    if example["correct"]:  # SEACrowd only use correct data
                        i += 1
                        yield str(i), {
                            "id": str(i),
                            "text_1": example["en_segment"],
                            "text_2": example["th_segment"],
                            "text_1_name": "eng",
                            "text_2_name": "tha",
                        }



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@misc{feryandi2018,
  author={Nurdiantoro, Feryandi}
  title={Dataset-Artikel},
  year = {2018},
  url = {https://github.com/feryandi/Dataset-Artikel},
}
"""

_DATASETNAME = "id_newspaper_2018"

_DESCRIPTION = """\
The ID Newspapers 2018 dataset provides 500K articles from various Indonesian news sources. Articles were taken from
7 primary sources (Detik, Kompas, Tempo, CNN Indonesia, Sindo, Republika, Poskota). The compressed files can be
retrieved from datahttps://huggingface.co/datasets/indonesian-nlp/id_newspapers_2018.
"""

_HOMEPAGE = "https://github.com/feryandi/Dataset-Artikel"

_LANGUAGES = ["ind"]

_LICENSE = Licenses.CC_BY_SA_4_0.value

_LOCAL = False

_URLS = "https://huggingface.co/datasets/indonesian-nlp/id_newspapers_2018/resolve/main/newspapers-json.tgz"

_SUPPORTED_TASKS = [Tasks.SELF_SUPERVISED_PRETRAINING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class IDNewspapers2018Dataset(datasets.GeneratorBasedBuilder):
    """
    ID Newspapers 2018 is a pretraining dataset from https://huggingface.co/datasets/indonesian-nlp/id_newspapers_2018.
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=datasets.Version(_SOURCE_VERSION),
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_ssp",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_ssp",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features({"url": datasets.Value("string"), "date": datasets.Value("string"), "title": datasets.Value("string"), "content": datasets.Value("string")})
        elif self.config.schema == "seacrowd_ssp":
            features = schemas.ssp_features
        else:
            raise ValueError(f"Invalid schema: '{self.config.schema}'")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """
        Returns SplitGenerators.
        """

        path = dl_manager.download_and_extract(_URLS)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "path": path,
                    "split": "train",
                },
            )
        ]

    def _generate_examples(self, path: Path, split: str) -> Tuple[int, Dict]:
        """
        Yields examples as (key, example) tuples.
        """
        file_paths = []
        for path, subdirs, files in os.walk(path):
            for name in files:
                if name[-5:] == ".json":
                    file_paths.append(os.path.join(path, name))

        for idx, file_path in enumerate(file_paths):
            with open(file_path, "r", encoding="utf-8") as file:
                data = json.load(file)

                if self.config.schema == "source":
                    x = {
                        "url": data["url"],
                        "date": data["date"],
                        "title": data["title"],
                        "content": data["content"],
                    }
                    yield idx, x

                elif self.config.schema == "seacrowd_ssp":
                    x = {
                        "id": str(idx),
                        "text": data["content"],
                    }
                    yield idx, x

                else:
                    raise ValueError(f"Invalid schema: '{self.config.schema}'")



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@inproceedings{bunagtransformer,
  author={Bunag, Kenrick Lance T and Esquivel, Rosanna A}
  title={Transformer-Based Conditional Language Models to Generate Filipino News Articles},
  year = {2023},
  publisher = {IEOM Society International},
  url = {https://ieomsociety.org/proceedings/2023manila/595.pdf},
  booktitle = {Proceedings of the International Conference on Industrial Engineering and Operations Management},
  pages = {2231–2237},
  numpages = {7},
  location = {Manila, Philippines},
}
"""

_DATASETNAME = "balita_nlp"

_DESCRIPTION = """\
BalitaNLP is a dataset for image-conditional language generation and text-conditional image generation. It consists of 300k Filipino news
articles and images gathered from Filipino news outlets. News articles are categorized into five possible classes: News, Sports, Entertainment,
Crime, and Other. Some articles were removed from the SEACrowd `imtext` schema, as their corresponding image files do not exist:
- `train` split (262480 total articles): from the original 281403 articles, 18923 (~6.72%) had missing images
- `test` split (32821 total articles): from the original 35177 articles, 2356 (~6.70%) had missing images
- `validation` split (32806 total articles): from the original 35175 articles, 2369 (~6.73%) had missing images
"""

_HOMEPAGE = "https://github.com/KenrickLance/BalitaNLP-Dataset"

_LANGUAGES = ["fil"]

_LICENSE = Licenses.UNKNOWN.value

_LOCAL = False

_URLS = {
    "text": "https://storage.googleapis.com/public-kenricklancebunag/BalitaNLP/2022/BalitaNLP-Dataset.zip",
    "images": {
        "part1": "https://storage.googleapis.com/public-kenricklancebunag/BalitaNLP/2022/BalitaNLP-images_1.zip",
        "part2": "https://storage.googleapis.com/public-kenricklancebunag/BalitaNLP/2022/BalitaNLP-images_2.zip",
        "part3": "https://storage.googleapis.com/public-kenricklancebunag/BalitaNLP/2022/BalitaNLP-images_3.zip",
        "part4": "https://storage.googleapis.com/public-kenricklancebunag/BalitaNLP/2022/BalitaNLP-images_4.zip",
    },
}

_SUPPORTED_TASKS = [Tasks.IMAGE_CAPTIONING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class BalitaNLPDataset(datasets.GeneratorBasedBuilder):
    """
    BalitaNLP is an image-text dataset from https://github.com/KenrickLance/BalitaNLP-Dataset.
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=datasets.Version(_SOURCE_VERSION),
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_imtext",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_imtext",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "body": datasets.Sequence(datasets.Value("string")),
                    "title": datasets.Value("string"),
                    "website": datasets.Value("string"),
                    "category": datasets.Value("string"),
                    "date": datasets.Value("string"),
                    "author": datasets.Value("string"),
                    "url": datasets.Value("string"),
                    "img_url": datasets.Value("string"),
                    "img_path": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_imtext":
            features = schemas.image_text_features()
            features["metadata"] = {
                "context": datasets.Value("string"),
                "author": datasets.Value("string"),
                "category": datasets.Value("string"),
                "date": datasets.Value("string"),
                "img_url": datasets.Value("string"),
                "url": datasets.Value("string"),
                "website": datasets.Value("string"),
            }
        else:
            raise ValueError(f"Invalid schema: '{self.config.schema}'")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """
        Returns SplitGenerators.
        """

        text_path = dl_manager.download_and_extract(_URLS["text"])
        img_paths = dl_manager.download_and_extract([v for k, v in _URLS["images"].items()])

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "text_path": os.path.join(text_path, "train.json"),
                    "img_paths": img_paths,
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "text_path": os.path.join(text_path, "test.json"),
                    "img_paths": img_paths,
                    "split": "test",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "text_path": os.path.join(text_path, "validation.json"),
                    "img_paths": img_paths,
                    "split": "validation",
                },
            ),
        ]

    def _generate_examples(self, text_path: Path, img_paths: Path, split: str) -> Tuple[int, Dict]:
        """
        Yields examples as (key, example) tuples.
        """
        text_data = pd.read_json(text_path)
        data = text_data.to_records()

        for idx, row in enumerate(data):

            # Search for path of image file
            img_path = ""
            for idx_subpath, img_subpath in enumerate(img_paths):
                candidate_filepath = os.path.join(img_subpath, "part" + str(idx_subpath + 1), row["img_path"])
                if os.path.isfile(candidate_filepath):
                    img_path = candidate_filepath

            if self.config.schema == "source":
                x = {
                    "body": row["body"],
                    "title": row["title"],
                    "website": row["website"],
                    "category": row["category"],
                    "date": row["date"],
                    "author": row["author"],
                    "url": row["url"],
                    "img_url": row["img_url"],
                    "img_path": img_path,
                }
                yield idx, x

            elif self.config.schema == "seacrowd_imtext":

                # Remove examples with no existing image path
                if img_path == "":
                    continue

                x = {
                    "id": idx,
                    "image_paths": [img_path],
                    "texts": row["title"],
                    "metadata": {
                        "context": row["body"],
                        "author": row["author"],
                        "category": row["category"],
                        "date": row["date"],
                        "img_url": row["img_url"],
                        "url": row["url"],
                        "website": row["website"],
                    },
                }
                yield idx, x

            else:
                raise ValueError(f"Invalid schema: '{self.config.schema}'")



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import requests

from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import SCHEMA_TO_FEATURES, TASK_TO_SCHEMA, Licenses, Tasks

_CITATION = r"""\
@inproceedings{tiedemann-2012-parallel,
    title = "Parallel Data, Tools and Interfaces in {OPUS}",
    author = {Tiedemann, J{\"o}rg},
    editor = "Calzolari, Nicoletta  and
        Choukri, Khalid  and
        Declerck, Thierry  and
        Do{\u{g}}an, Mehmet U{\u{g}}ur  and
        Maegaard, Bente  and
        Mariani, Joseph  and
        Moreno, Asuncion  and
        Odijk, Jan  and
        Piperidis, Stelios",
    booktitle = "Proceedings of the Eighth International Conference on Language
    Resources and Evaluation ({LREC}'12)",
    month = may,
    year = "2012",
    address = "Istanbul, Turkey",
    publisher = "European Language Resources Association (ELRA)",
    url = "http://www.lrec-conf.org/proceedings/lrec2012/pdf/463_Paper.pdf",
    pages = "2214--2218",
    abstract = "This paper presents the current status of OPUS, a growing
    language resource of parallel corpora and related tools. The focus in OPUS
    is to provide freely available data sets in various formats together with
    basic annotation to be useful for applications in computational linguistics,
    translation studies and cross-linguistic corpus studies. In this paper, we
    report about new data sets and their features, additional annotation tools
    and models provided from the website and essential interfaces and on-line
    services included in the project.",
}
"""

_DATASETNAME = "gnome"

_DESCRIPTION = """\
A parallel corpus of GNOME localization files, which contains the interface text
in the GNU Network Object Model Environment (GNOME) and published by GNOME
translation teams. Text in this dataset is relatively short and technical.
"""

_HOMEPAGE = "https://opus.nlpl.eu/GNOME/corpus/version/GNOME"

_LANGUAGES = ["eng", "vie", "mya", "ind", "tha", "tgl", "zlm", "lao"]
_SUBSETS = ["en", "vi", "my", "id", "th", "tl", "ms", "lo"]
_SUBSET_PAIRS = [(src, tgt) for src in _SUBSETS for tgt in _SUBSETS if src != tgt]

_LICENSE = Licenses.UNKNOWN.value

_LOCAL = False

_URLS = {
    "api": "http://opus.nlpl.eu/opusapi/?source={src_lang}&target={tgt_lang}&corpus=GNOME&version=v1",
    "data": "https://object.pouta.csc.fi/OPUS-GNOME/v1/moses/{lang_pair}.txt.zip",
}

_SUPPORTED_TASKS = [Tasks.MACHINE_TRANSLATION]
_SEACROWD_SCHEMA = f"seacrowd_{TASK_TO_SCHEMA[_SUPPORTED_TASKS[0]].lower()}"  # t2t

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class GnomeDataset(datasets.GeneratorBasedBuilder):
    """A parallel corpus of GNOME localization files"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = []
    for subset in _SUBSET_PAIRS:
        lang_pair = f"{subset[0]}-{subset[1]}"
        BUILDER_CONFIGS += [
            SEACrowdConfig(
                name=f"{_DATASETNAME}_{lang_pair}_source",
                version=SOURCE_VERSION,
                description=f"{_DATASETNAME} {lang_pair} source schema",
                schema="source",
                subset_id=lang_pair,
            ),
            SEACrowdConfig(
                name=f"{_DATASETNAME}_{lang_pair}_{_SEACROWD_SCHEMA}",
                version=SEACROWD_VERSION,
                description=f"{_DATASETNAME} {lang_pair} SEACrowd schema",
                schema=_SEACROWD_SCHEMA,
                subset_id=lang_pair,
            ),
        ]

    DEFAULT_CONFIG_NAME = (
        f"{_DATASETNAME}_{_SUBSET_PAIRS[0][0]}-{_SUBSET_PAIRS[0][1]}_source"
    )

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "source": datasets.Value("string"),
                    "target": datasets.Value("string"),
                }
            )
        elif self.config.schema == _SEACROWD_SCHEMA:
            features = SCHEMA_TO_FEATURES[
                TASK_TO_SCHEMA[_SUPPORTED_TASKS[0]]
            ]  # text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        src_lang, tgt_lang = self.config.subset_id.split("-")
        api_url = _URLS["api"].format(src_lang=src_lang, tgt_lang=tgt_lang)
        data_url = None

        response = requests.get(api_url, timeout=10)
        if response:
            corpora = response.json()["corpora"]
            for corpus in corpora:
                if ".txt.zip" in corpus["url"]:
                    data_url = corpus["url"]
                    break
        else:
            raise requests.exceptions.HTTPError(
                f"Non-success status code: {response.status_code}"
            )

        if not data_url:
            raise ValueError(f"No suitable corpus found, check {api_url}")
        else:
            lang_pair = data_url.split("/")[-1].split(".")[0]
            data_dir = Path(dl_manager.download_and_extract(data_url))
            src_file = data_dir / f"GNOME.{lang_pair}.{src_lang}"
            tgt_file = data_dir / f"GNOME.{lang_pair}.{tgt_lang}"

            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    gen_kwargs={
                        "src_file": src_file,
                        "tgt_file": tgt_file,
                    },
                ),
            ]

    def _generate_examples(self, src_file: Path, tgt_file: Path) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        with open(src_file, "r", encoding="utf-8") as src_f, open(
            tgt_file, "r", encoding="utf-8"
        ) as tgt_f:
            for idx, (src_line, tgt_line) in enumerate(zip(src_f, tgt_f)):
                if self.config.schema == "source":
                    yield idx, {"source": src_line.strip(), "target": tgt_line.strip()}
                elif self.config.schema == _SEACROWD_SCHEMA:
                    yield idx, {
                        "id": str(idx),
                        "text_1": src_line.strip(),
                        "text_2": tgt_line.strip(),
                        "text_1_name": f"source ({src_file.name.split('.')[-1]})",
                        "text_2_name": f"target ({tgt_file.name.split('.')[-1]})",
                    }
